classdef Mcam < handle
        
    properties
    	numberFrameDisplayPerSecond = 10;   % Frames per segon.
    	vid;                                % Objecte dispositiu de captura.
    	timerData;                          % Dades del timer.
    end
    
    methods
        
        %   iniWebCam - Inicialitza l'objecte Mcam
        %
        %   Usage:      cam = Mcam();
        %               ret = cam.iniWebCam();
        %
        %   Arguments:
        %               none.
        %
        %   Returns:
        %               ret - cert si la inicialitzaci ha funcionat b. Fals, altrament.
        %
    	function [ret] = iniWebCam(obj)        
    		ret = true;
    		try
   				% For windows
   				obj.vid = videoinput('winvideo', 1);
   				% Set parameters for video
				% Acquire only one frame each time
				set(obj.vid,'FramesPerTrigger',1);
				% Go on forever until stopped
				set(obj.vid,'TriggerRepeat',Inf);
				% Get a rgb image
				set(obj.vid,'ReturnedColorSpace','rgb');
				triggerconfig(obj.vid, 'Manual');
			catch
      			ret = false;
			end
        end

        %   realVideo - Crida a la funci frameDisplay per mostrar la
        %               imatge capturada en els axes handlesAxes, depenent
        %               del codi d'operaci sobre la imatge s'hi pot
        %               practicar el reconeixement o la detecci.
        %               Es crida el timer cada x temps per capturar el
        %               contingut de la webcam
        %
        %   Usage:      cam = Mcam();
        %               cam.realVideo(handlesAxes,op)
        %
        %   Arguments:
        %               handleAxes - Axes del gui on es mostra la imatge
        %               op - Codi operaci: 
        %                       Params.recEigenVid(es mostra la imatge i es fa el reconeixement amb Eigenfaces)             
        %                       Params.recFisherVid(es mostra la imatge i es fa el reconeixement amb Fisherfaces)
        %                       Params.preview(noms mostra la imatge de la webcam)
        %                       Params.recEigenEst(es mostra la imatge i es fa el reconeixement amb Eigenfaces sobre la imatge esttica)
        %                       Params.recEigenEst(es mostra la imatge i es fa el reconeixement amb Fisherfaces sobre la imatge esttica)
        %
        %   Returns:
        %               none.
        %        
    	function realVideo(obj,handleAxes,op)
    		
            p = load('params.mat','Params');
            Params = p.Params;
            
            % set up timer object
			if strcmp(op,Params.recEigenVid) == 1 || strcmp(op,Params.preview) == 1 || strcmp(op,Params.recFisherVid) == 1
                % Les captures de frames es fan utilitzant un timer
                obj.timerData=timer('TimerFcn', {@frameDisplay,obj.vid,handleAxes,op},'Period',1/obj.numberFrameDisplayPerSecond,'ExecutionMode','fixedRate','BusyMode','drop');
            elseif strcmp(op,Params.recEigenEst) == 1 || strcmp(op,Params.recFisherEst) == 1
                % Per imatges estàtiques
                frameDisplay(NaN, NaN, NaN, handleAxes, op);
            end
        end
        
        %   realVideo2 - Funci especfica pel formulari d'alta d'una imatge 
        %                a la base de dades. Igual que realVideo.
        %                Es crida el timer cada x temps per capturar el
        %                contingut de la webcam
        %
        %   Usage:      cam = Mcam();
        %               cam.realVideo2(handles,axes)
        %
        %   Arguments:
        %               handles - Handles del gui
        %               src - Axes a on es mostra la imatge: 
        %                       Params.video(La imatge origen s vdeo de la webcam)             
        %                       Params.photo(La imatge origen s esttica)        
        %
        %   Returns:
        %               none.
        % 
        function realVideo2(obj,handles, src)
            
            p = load('params.mat','Params');
            Params = p.Params;
            
            % set up timer object
            if strcmp(src,Params.video) == 1
                % Les captures de frames es fan utilitzant un timer
                obj.timerData=timer('TimerFcn', {@frameDisplay2,obj.vid,handles, src},'Period',1/obj.numberFrameDisplayPerSecond,'ExecutionMode','fixedRate','BusyMode','drop');
            elseif strcmp(src,Params.photo) == 1
                % Per imatges estàtiques
                frameDisplay2(NaN, NaN, NaN, handles, src);
            end
        end

        %   startWebCamCapture - Inicia la captura de la webcam        
        %
        %   Usage:      cam = Mcam();
        %               cam.startWebCamCapture()
        %
        %   Arguments:
        %               none.
        %
        %   Returns:
        %               none.
        % 
    	function startWebCamCapture(obj)
    		start(obj.vid);
    		start(obj.timerData);
        end

        %   stopWebCamCapture - Atura la captura de la webcam        
        %
        %   Usage:      cam = Mcam();
        %               cam.stopWebCamCapture()
        %
        %   Arguments:
        %               none.
        %
        %   Returns:
        %               none.
        % 
    	function stopWebCamCapture(obj)
    		stop(obj.vid);
			delete(obj.vid);
        end

        %   startTimer - Atura el timer        
        %
        %   Usage:      cam = Mcam();
        %               cam.stopTimer()
        %
        %   Arguments:
        %               none.
        %
        %   Returns:
        %               none.
        %  
    	function stopTimer(obj)
    		stop(obj.timerData);
			delete(obj.timerData);			
    	end
	end
            
end

