classdef MfaceDetect
        
    methods (Static)
                
        % 	doFaceAndEyeDetect - Detecta cara i ulls en una imatge
        %
        %   Usage:      [faces, numFaces, eyes] = doFaceAndEyeDetect(img,scale, pBoxEyeR, pBoxEyeL)
        %
        %   Arguments:
        %               img     Imatge sobre la que aplicar la deteccio de cares (en escala de grisos)
        %               scale   Parametre escala de FaceDetect
        %
        %   Returns:
        %               faces       Rectangles de cares
        %               numFaces    Nombre de cares trobades
        %               eyes        Rectangles dels ulls
        %                 
        function [faces, numFaces, eyes] = doFaceAndEyeDetect(img, scale, pBoxEyeR, pBoxEyeL)
            % Necessari per poder utilitzar viola&jones
            setenv('KMP_DUPLICATE_LIB_OK','TRUE');
            
            p = load('params.mat','Params');
            Params = p.Params;
            
            % Inicializacions
            xml = Params.xmlHaarFace;            
            faces = [];
            eyes = [];
            numFaces = 0;            
            
            % Crida a viola jones (mex)
            detectedFaces = FaceDetect(xml,img,scale);
            numFaces = size(detectedFaces,1);
            
            %Deteccio estimada (si la cara esta torta ja no funciona....)
            if detectedFaces ~=-1                
                faces = detectedFaces;
                for h=1:size(detectedFaces)
                    siz = detectedFaces(h,3);
                                        
                    % Bounding box ull esquerre
                    eyes = [eyes; round([detectedFaces(h,1)+siz*pBoxEyeL(1),detectedFaces(h,2)+siz*pBoxEyeL(3),siz*(1-pBoxEyeL(2)-pBoxEyeL(1)),siz*(1-pBoxEyeL(3)-pBoxEyeL(4))])];                                        
                    
                    % Bounding box ull dret
                    eyes = [eyes; round([detectedFaces(h,1)+siz*pBoxEyeR(1),detectedFaces(h,2)+siz*pBoxEyeR(3),siz*(1-pBoxEyeR(2)-pBoxEyeR(1)),siz*(1-pBoxEyeR(3)-pBoxEyeR(4))])];                                        
                end
            else
                numFaces = 0;
            end
        end        
    
        %   doFaceDetect - Detecta cara en una imatge
        %
        %   Usage:      [faces, numFaces] = doFaceDetect(img, scale) 
        %
        %   Arguments:
        %               img     Imatge sobre la que aplicar la deteccio de cares (en escala de grisos)
        %               scale   Parametre escala de FaceDetect
        %
        %   Return:
        %           faces       Rectangles de cares
        %           numFaces    Nombre de cares trobades
        %        
        function [faces, numFaces] = doFaceDetect(img, scale)
            % Necessari per poder utilitzar viola&jones
            setenv('KMP_DUPLICATE_LIB_OK','TRUE');
            
            p = load('params.mat','Params');
            Params = p.Params;

            % Inicializacions
            xml = Params.xmlHaarFace;            
            faces = [];            
            numFaces = 0;            

            % Crida a viola jones (mex)
            detectedFaces = FaceDetect(xml,img,scale);
            numFaces = size(detectedFaces,1);

            if detectedFaces ~=-1                
                faces = detectedFaces;    
            else
                numFaces = 0;
            end
        end
        
        %   doEyeDetect - Detecta ulls en una imatge
        %
        %   Usage:  [eyes, numEyes] = doEyeDetectVj(img, scale)
        %
        %   Arguments:
        %               img     Imatge sobre la que aplicar la deteccio de cares (en escala de grisos)
        %               scale   Parametre escala de FaceDetect
        %
        %   Return:
        %           eyes      Rectangles de cares
        %           numEyes   Nombre de cares trobades
        %  
        function [eyes, numEyes] = doEyeDetectVj(img, scale)
            % Necessari per poder utilitzar viola&jones
            setenv('KMP_DUPLICATE_LIB_OK','TRUE');
            
            p = load('params.mat','Params');
            Params = p.Params;

            % Inicializacions
            xml = Params.xmlHaarEye;            
            eyes = [];            
            numEyes = 0;            

            % Crida a viola jones (mex)
            detectedEyes = EyeDetect(xml,img,scale);
            numEyes = size(detectedEyes,1);
        
            if detectedEyes ~=-1                
                eyes = detectedEyes;
            else
                numEyes = 0;
            end
        end
        
        % -----------------------------------------------------------------
        % Input params:
        % img               
        %
        % Output params:
        % ulls
        % -----------------------------------------------------------------
        
        function doMorphAnalisys(img)
            
        end
        
    end
    
    methods (Static, Access = private)
        % -----------------------------------------------------------------
        % Input params:
        % img               
        %
        % Output params:
        % ulls
        % -----------------------------------------------------------------
        
        function [centroids] = doEyeDetectMorphRgb(img)
                        
            y = img(:,:,1);
            cb = double(img(:,:,2));
            cr = double(img(:,:,3));
            cb2 = cb.*cb;
            cr_2 = (255-cr).*(255-cr);
            
            e = 1/3*(cb2+cr_2+cb./cr);
            en = e-min(e(:));
            en = en/max(en(:));
            
            x =sum(en(:))/(size(en, 1) * size(en, 2))*1.25;
            if x<0.54
                x=0.54;
            end
            EyeMapC = en>x;
            
            yd = strel('ball', 5, size(en, 1)*0.12);
            ydd= imdilate(y, yd);
            yee = imerode(y, yd)+1;
            EML = (ydd./yee);
            EML1 = EML-min(EML(:));
            EML1 = EML1/max(EML1(:));
            EyeMapL = EML1>x;
            
            EyeMap = and(EyeMapC, EyeMapL);
            figure,imshow(EyeMap);
                        
            se = strel('disk',round(max(size(en))/300));
            [L,n] = bwlabel(imdilate(EyeMap, se, 'full'));
            
            % Noms cal tenir en compte la meitat superior del rostre
            L2 = L(1:size(L, 1)/2, 1:size(L, 2));
            stats = regionprops(L2, 'centroid');
            
            % Obtencio dels centroides 
            centroids = cat(1, stats.Centroid);            
            centroids(any(isnan(centroids),2),:)=[];
                        
            % Ordenar el vector de centroides en funci de les x (esquerra a dreta)            
            centroids = sortrows(centroids);                                    
        end        
        
        % -----------------------------------------------------------------
        % Input params:
        % img               
        %
        % Output params:
        % ulls
        % -----------------------------------------------------------------
        
        function [centroids] = doMouthDetectMorphRgb(img)
            
            cb= double(img(:,:,2));
            cr = double(img(:,:,3));
            cr2 = cr.*cr;
            cr2n = cr2-min(cr2(:));
            cr2n = cr2n/max(cr2n(:));
            uno = cr2n>0.53;
            
            co = sum(sum(cr2))/(size(cr, 2)*size(cr, 1));
            di = (sum(sum(cr))/sum(sum(cb))*1/(size(cr, 2)*size(cr, 1)));
            n=0.95*co/di;
            
            a = cr2;
            an = a -min(a(:));
            an = an/max(an(:));
            za = an>0.8;
            b = n*cr./cb;
            bn = b-min(b(:));
            bn = bn/max(bn(:));
            zb = bn>0.55;

            dos = xor(za, zb);
            MountMapn= and(uno, dos);
            se = strel('disk',round(max(size(img))/300));
            [L,n] = bwlabel(imdilate(MountMapn, se, 'full'));
                       
            stats = regionprops(L, 'centroid');

            % Obtencio dels centroides 
            centroids = cat(1, stats.Centroid);            
            centroids(any(isnan(centroids),2),:)=[];
                        
            % Ordenar el vector de centroides en funci de les x (esquerra a dreta)            
            centroids = sortrows(centroids);  
        end
    end
end

