classdef MfaceRecognition
        
    methods (Static)
        
        % -----------------------------------------------------------------
        % Input params:
        % newMat
        % maxImgs
        % databasePath  
        % databasePathMat
        %
        % Output params:
        % w
        % noms        
        % -----------------------------------------------------------------
        
        function [w, noms] = loadTestDatabase(newMat, testDatabasePath, databasePathMat)            
            if newMat
                v=[];
                noms = [];
                fList = Utils.getSubFolders(testDatabasePath);
                for i=1:size(fList,2)                        
                    iList = Utils.getImages(strcat(testDatabasePath, '\', sprintf('%s',fList{i})));
                    for j=1:maxImgs        
                        % Ha d'estar en escala de grisos                                        
                        a=imread(strcat(databasePath, '\', sprintf('%s',fList{i}), '\', sprintf('%s',iList{j})));
                        v = [v reshape(a,size(a,1)*size(a,2),1)];
                        noms = [noms fList(i)];
                    end                   
                end
                w=uint8(v);
                save(databasePathMat,'w','noms');
            else
                lw = load(databasePathMat,'w');
                lnoms = load(databasePathMat,'noms');
                w = lw.w;
                noms = lnoms.noms;
            end                        
        end

        % -----------------------------------------------------------------
        % Input params:
        % newMat
        % maxImgs
        % databasePath  
        % databasePathMat
        %
        % Output params:
        % w
        % noms        
        % -----------------------------------------------------------------
        
        function [w, noms] = loadDatabasePca(newMat, maxImgs, databasePath, databasePathMat)            
            if newMat
                v=[];
                noms = [];
                fList = Utils.getSubFolders(databasePath);
                for i=1:size(fList,2)                        
                    iList = Utils.getImages(strcat(databasePath, '\', sprintf('%s',fList{i})));
                    for j=1:maxImgs        
                        % Ha d'estar en escala de grisos                                        
                        a=imread(strcat(databasePath, '\', sprintf('%s',fList{i}), '\', sprintf('%s',iList{j})));
                        v = [v reshape(a,size(a,1)*size(a,2),1)];
                        noms = [noms fList(i)];
                    end                   
                end
                w=uint8(v);
                save(databasePathMat,'w','noms');
            else
                lw = load(databasePathMat,'w');
                lnoms = load(databasePathMat,'noms');
                w = lw.w;
                noms = lnoms.noms;
            end                        
        end
        
        % -----------------------------------------------------------------
        % Input params:
        % newMat
        % maxImgs
        % databasePath  
        % databasePathMat
        %
        % Output params:
        % w
        % noms        
        % -----------------------------------------------------------------
        
        function [w, noms] = loadDatabaseLda(newMat, maxImgs, databasePath, databasePathMat)            
            if newMat
                v=[];
                noms = [];
                fList = Utils.getSubFolders(databasePath);
                for i=1:size(fList,2)                        
                    iList = Utils.getImages(strcat(databasePath, '\', sprintf('%s',fList{i})));
                    for j=1:maxImgs        
                        % Ha d'estar en escala de grisos                                        
                        a=imread(strcat(databasePath, '\', sprintf('%s',fList{i}), '\', sprintf('%s',iList{j})));
                        v = [v reshape(a',size(a,1)*size(a,2),1)];
                        noms = [noms fList(i)];
                    end                   
                end
                w=uint8(v);
                save(databasePathMat,'w','noms');
            else
                lw = load(databasePathMat,'w');
                lnoms = load(databasePathMat,'noms');
                w = lw.w;
                noms = lnoms.noms;
            end                        
        end
        
        % -----------------------------------------------------------------
        % Input params:
        % testImage
        % Nev        
        %
        % Output params:
        % idx
        % dist
        % imatge
        % errorMida
        % -----------------------------------------------------------------
        
        function [idx, dist, imatge, errorMida] = pcaFaceRecognition(testImage, Nev, trainBdd)                       
                        
            r = uint8(reshape(testImage,size(testImage,1)*size(testImage,2),1)); 
            errorMida = true;
            idx=[];
            dist=0;
            imatge=[];
            
            % La mida de les imatges ha de ser la mateixa
            if size(r,1)==size(trainBdd,1)
                errorMida = false;
                v = trainBdd;           

                N=Nev;                              

                O=uint8(ones(1,size(v,2))); 
                m=uint8(mean(v,2));                 % m is the maen of all images.
                vzm=v-uint8(single(m)*single(O));   % vzm is v with the mean removed. 


                L=single(vzm)'*single(vzm);
                [V,D]=eig(L);
                V=single(vzm)*V;
                V=V(:,end:-1:end-(N-1));            % Pick the eignevectors corresponding to the 10 largest eigenvalues. 

                cv=zeros(size(v,2),N);

                for i=1:size(v,2);
                    cv(i,:)=single(vzm(:,i))'*V;    % Each row in cv is the signature for one image.
                end

                p=r-m;                              % Subtract the mean
                s=single(p)'*V;
                z=[];

                for i=1:size(v,2)
                    z=[z,norm(cv(i,:)-s,2)];            
                end

                [a,i]=min(z);
                idx = i;
                dist = a;
                imatge = reshape(v(:,i),size(testImage,1),size(testImage,2));
            end
        end
        
        % -----------------------------------------------------------------
        % Input params:
        % testImage
        % Nev        
        %
        % Output params:
        % idx
        % dist
        % imatge
        % errorMida
        % -----------------------------------------------------------------
        
        function [idx, dist, imatge, errorMida] = ldaFaceRecognition(testImage, numperclass, trainBdd)

            idx = [];
            dist = 0;
            imatge = [];
            errorMida = true;
           
            testImage = double(testImage);            
            r = reshape(testImage',size(testImage,1)*size(testImage,2),1);
                                   
            if size(r,1) == size(trainBdd,1)
                errorMida = false;
                [Psi V_KLT V_FLD Proj_fisher] = MfaceRecognition.fisherface(trainBdd,numperclass);
                [dist, idx] = min(MfaceRecognition.fisher_dist(r,Psi,V_KLT,V_FLD,Proj_fisher));

                imatge = reshape(trainBdd(:,idx),size(testImage,2),size(testImage,1));
                imatge = imatge';
            end
        end
    end
    
    methods (Static, Access = private)
       
        function [Psi V_KLT V_FLD Proj_fisher] = fisherface(T,numperclass)
            
            T = double(T);
            numimg = size(T,2);
            numclass = numimg/numperclass; % Number of classes (or persons)
            numvec = numimg-numclass;
            [V_KLT,~,Psi,A] = MfaceRecognition.evectors(T,numvec);
            V_KLT = V_KLT./repmat(sum(V_KLT.^2).^0.5,size(V_KLT,1),1);
            Proj_eig = V_KLT'* A;

            % Calculating the mean of each class in eigenspace
            m_Proj = mean(Proj_eig,2); % Total mean in eigenspace
            m = zeros(numvec,numclass); % Class mean in eigenspace
            Rw = zeros(numvec,numvec); % Initialization of Within Scatter Matrix
            Rb = zeros(numvec,numvec); % Initialization of Between Scatter Matrix

            for i = 1 : numclass
                m(:,i) = mean( ( Proj_eig(:,((i-1)*numperclass+1):i*numperclass) ), 2 )';    
                R  = zeros(numvec,numvec); 
                for j = ( (i-1)*numperclass+1 ) : ( i*numperclass )
                    R = R + (Proj_eig(:,j)-m(:,i))*(Proj_eig(:,j)-m(:,i))';
                end
                Rw = Rw + R; % Within Scatter Matrix
                Rb = Rb + (m(:,i)-m_Proj) * (m(:,i)-m_Proj)'; % Between Scatter Matrix
            end

            % Calculating Fisher discriminant basis's
            [Vtemp, Dtemp] = eig(Rb,Rw); % inv(Rw) * Rb
            Dtemp = abs(diag(Dtemp));
            [~, sort_idx] = sort(Dtemp,'descend');            
            int_idx = int8(round(sort_idx));
            V_FLD = Vtemp(:,int_idx);
            V_FLD = V_FLD(:,1:numclass-1);
            V_FLD = V_FLD./repmat(sum(V_FLD.^2).^0.5,size(V_FLD,1),1);

            % Projecting images onto Fisher linear space
            Proj_fisher = V_FLD'*Proj_eig; 
        end
       
        function [Vector,Value,Psi,A] = evectors(T,numvecs)

            % Check arguments
            if nargin ~= 2
                error('wrong number of input arguments');
            end;
            numimg = size(T,2);

            % Compute the "average" vector
            Psi = mean(T,2);

            % Compute difference with average for each vector
            A = T - repmat(Psi,1,numimg);

            % Covariance matrix
            L = A'*A;
            [V D] = eig(L);

            % Sort and convert the eigenvectors of A'*A into eigenvectors of A*A'
            D = abs(diag(D));
            [Value sort_idx] = sort(D,'descend');
            V = V(:,sort_idx);
            V = V(:,1:numvecs);
            Vector = A*V;
        end
        
        function D = fisher_dist(testImage,Psi,V_KLT,V_FLD,Proj_fisher)

            % TESTIMAGE is a grayscale image to be recognized.
            % PSI, V_KLT, V_FLD, PROJ_FISHER are outputs from the function
            % FISHERFACE_MAIN, which calculates the generalized eigenvectors from the
            % training set images.
            % [M,N] is the size of the training set images. 

            Proj_test = V_FLD'*V_KLT'*(double(testImage)-Psi);
            D = MfaceRecognition.distance(Proj_test,Proj_fisher);            
        end
        
        function D = distance(Proj1,Proj2)
            %
            % DISTANCE(PROJ1,PROJ2) calculates the distance matrix D, whose
            % element Dij is the distance between the ith column vector in PROJ1
            % and the jth column vector in PROJ2
            %

            numimgs1 = size(Proj1,2);
            numimgs2 = size(Proj2,2);
            n = size(Proj1,1);
            D = zeros(numimgs1,numimgs2);
            for i = 1:numimgs2
                D(:,i) = sqrt(sum((Proj1-Proj2(:,i)*ones(1,numimgs1)).^2))'/n;
            end
        end
    end
    
end

