classdef Utils
   
    methods (Static)
        
        % -----------------------------------------------------------------
        % Converteix una imatge a escala de grisos
        % Input params:
        % img       Imatge a convertir
        %
        % Output params:
        % retImg    Imatge convertida a escala de grisos
        % -----------------------------------------------------------------
        
        function retImg = doGray(img)
            %if RGB image make gray scale
            if (size(img,3)>1)
                try
                    %image toolbox dependent
                    img=double(rgb2gray(img));
                catch err
                    %if no image toolbox do simple sum
                    img=double(sum(double(img),3)/3);   
                end
            end
            
            if not(strcmp(class(img),'double'))
                img = double(img);
            end
            
            retImg = img;
        end
        
        % -----------------------------------------------------------------
        % Calcula el centre d'un rectangle
        % Input params:
        % rect Rectangle (centre inferior esquerre, alada, amplada)
        %
        % Output params:
        % x         Coordenada x centre
        % y         Coordenada y centre
        % -----------------------------------------------------------------
        
        function [x y] = rectangleCenter(rect)
            x = rect(1,1)+(rect(2,1)-rect(1,1))/2;
            y = rect(2,2)+(rect(3,2)-rect(2,2))/2;
        end
                
                
        % -----------------------------------------------------------------
        % Calcula la distncia entre dos punts
        % Input params:
        % p1        punt1
        % p2        punt2
        %
        % Output params:
        % dist      Distancia entre punts
        % -----------------------------------------------------------------
        
        function [ dist ] = distance( p1, p2 )
            dx = p2(1) - p1(1);
            dy = p2(2) - p1(2);
            dist = sqrt(dx*dx+dy*dy);
        end
        
        % -----------------------------------------------------------------
        % Calcula les subcarpetes d'un carpeta
        % Input params:
        % pathFolder carpeta arrel        
        %
        % Output params:
        % folders subcarpetes de la carpeta arrel      
        % -----------------------------------------------------------------
        
        function folders = getSubFolders(pathFolder)
            d =dir(pathFolder);
            isub = [d(:).isdir];
            folders = {d(isub).name};
            folders(ismember(folders,{'.','..'})) = [];
        end
        
        % -----------------------------------------------------------------
        % Retorna les imatges d'una carpeta
        % Input params:
        % pathFolder    ruta de la carpeta que cont les imatges       
        %
        % Output params:
        % images imatges que cont la carpeta (*.jpg i *.pgm)   
        % -----------------------------------------------------------------
        
        function images = getImages(pathFolder)           
            l1 = dir(strcat(pathFolder,'\*.jpg'));
            l2 = dir(strcat(pathFolder,'\*.pgm'));
            images = {l1(:).name l2(:).name};            
        end
        
        
        function dist = distancePointSegment(extrem1, extrem2, punt)
            dist = abs(det([extrem2-extrem1;punt-extrem1]))/norm(extrem2-extrem1); 
        end
        
        % -----------------------------------------------------------------
        % Pintar una imatge en un control axes
        % Input params:
        % axCaixaImg        control a on mostrar la imtge
        % rutaImg           ruta de la imatge
        %
        % Output params:
        % none.
        % -----------------------------------------------------------------
        
        function doImage(axCaixaImg, rutaImg)
            axes(axCaixaImg);
            cla;
            [x,map] = imread(rutaImg);
            imshow(x,map);
        end

        % -----------------------------------------------------------------
        % Crea un rectangle a partir d'una bounding box
        % Input params:
        % arr      bounding box  
        %
        % Output params:
        % r     rectangle   
        % -----------------------------------------------------------------

        function r = makeRect(arr)
            r = [arr(1,1) arr(1,2); 
                    arr(1,1)+arr(1,3) arr(1,2); 
                    arr(1,1)+arr(1,3) arr(1,2)+arr(1,4); 
                    arr(1,1)  arr(1,2)+arr(1,4); 
                    arr(1,1) arr(1,2)];
        end
    end
       
end

