% -----------------------------------------------------------------
% Reconeixement mitjanant eigenfaces de la imatge i 
% Input params:
% i  imatge d'entrada
%
% Output params:
% structCares Estructura que cont informaci de les cares detectades. Cada
% cella s una cara amb informaci (bbox, ulls, nom)
% -----------------------------------------------------------------

function [structCares] = eigenRecognition( i )

    p = load('params.mat','Params');
    Params = p.Params;

	% Escala minima dels procediments de detecció
	scaleFace = Params.scaleFaceMin;
	scaleEye = Params.scaleEyeMin;

	% Flags
	B_DETECTA_ULLS = false;
	B_FES_ALINEAMENT = false;
	B_FES_RECO = false;

	% Escala de resize dels crops
	scale = Params.scale;

	% Parametres reconeixement
	nTraingImg = Params.nTrainImgs;
	nEig = Params.nEigenVects;

	% Struct on guardar la info de les cares
	cares.cara = [];
	cares.ulls = [];
    cares.ue = [];
    cares.ud = [];
	cares.nom = '';
	cares.crop = [];

	%% Deteccio de les cares

	% Carrega de la imatge de prova i conversio a escala de grisos	
	iGray = Utils.doGray(i);

	[faces, numFaces] = MfaceDetect.doFaceDetect(iGray, scaleFace);

	if numFaces > 0	    
        B_DETECTA_ULLS = true;       
        for h=1:size(faces,1)
            cares(h).cara = faces(h,:); %requadre de la cara
            cares(h).ulls = [];
            cares(h).crop = [];
            cares(h).nom = 'unknown';
            cares(h).ue = [];
            cares(h).ud = [];
        end
    end
     
	%% Alineament de les cares

	% 1 - Detectar els ulls per fer l'alineament
	if B_DETECTA_ULLS 
	    
	    for h=1:size(faces,1)
	        %Es retalla la cara per poder-la alinear
	        iCrop = imcrop(i,[faces(h,1) faces(h,2) faces(h,3) faces(h,4)]);    
	        
	        %Es fa la imatge una mica més grossa per afavorir la detecció
	        iCropScaled = imresize(iCrop,scale);
	        iGrayScaled = Utils.doGray(iCropScaled);
	        
	        %Es busquen els ulls per poder fer l'alineament
	        [eyes, numEyes] = MfaceDetect.doEyeDetectVj(iGrayScaled, scaleEye);
	        
	        %Es guarda tots dins d'una estructura
	        if numEyes == 2                
	            %Es guarden els ulls	            
	            cares(h).ulls = eyes;
                cares(h).crop = [];
                cares(h).nom = 'unknown';
                cares(h).ue = [];
                cares(h).ud = [];
	            
	            B_FES_ALINEAMENT = true;
	        end        
	    end
	end

	% 2 - Un cop detectats els ulls es fa l'alineament i l'escalat de la imatge prenent l'ull esquerre 
	% com a referència.
	if B_FES_ALINEAMENT   
	    B_FES_RECO = true;
	   
        for h=1:size(cares,2)
	        face = cares(h).cara;
	        eyes = cares(h).ulls;
            
	        if size(eyes,1) == 2
                %Bounding box dels ulls
                bBoxUllDre = [eyes(1,1) eyes(1,2); 
                                eyes(1,1)+eyes(1,3) eyes(1,2); 
                                eyes(1,1)+eyes(1,3) eyes(1,2)+eyes(1,4); 
                                eyes(1,1)  eyes(1,2)+eyes(1,4); 
                                eyes(1,1) eyes(1,2)]/scale;

                bBoxUllEsq = [eyes(2,1) eyes(2,2); 
                                eyes(2,1)+eyes(2,3) eyes(2,2); 
                                eyes(2,1)+eyes(2,3) eyes(2,2)+eyes(2,4); 
                                eyes(2,1)  eyes(2,2)+eyes(2,4); 
                                eyes(2,1) eyes(2,2)]/scale;

                %Respecte la imatge sencera
                bBoxUllEsq = [face(1,1)+bBoxUllEsq(:,1) face(1,2)+bBoxUllEsq(:,2)];
                bBoxUllDre = [face(1,1)+bBoxUllDre(:,1) face(1,2)+bBoxUllDre(:,2)];
                
                cares(h).ue = bBoxUllEsq;
                cares(h).ud = bBoxUllDre;

                %Posicions dels ulls
                [xDabs yDabs] = Utils.rectangleCenter(bBoxUllDre);
                [xEabs yEabs] = Utils.rectangleCenter(bBoxUllEsq);
  
                %Rotar la imatge original
                [iRotated,xdata,ydata] = MfaceAlign.rotate(i,[xEabs yEabs],[xDabs yDabs]);   
                
                %Fer el crop
                cares(h).crop = MfaceAlign.cropFaceNoRotate(iRotated,[xEabs-xdata(1) yEabs-ydata(1)], [xDabs-xdata(1) yDabs-ydata(1)],[0.3 0.3], [Params.alcada Params.amplada]);
                cares(h).nom = 'unknown';
            end
	    end
	end

	%% Reconeixement

	if B_FES_RECO 
        
	    [w, noms] = MfaceRecognition.loadDatabasePca(false,nTraingImg,Params.pathDatabase, Params.pathDatabaseMatFilePca);
	    
	    for h=1:size(cares,2)
	        if size(cares(h).crop,1)>0 && size(cares(h).crop,2)>0
                iR = Utils.doGray(cares(h).crop);            
                [idx, dist, imatge, errorMida] = MfaceRecognition.pcaFaceRecognition(iR,nEig,w);

                if not(errorMida)	            
                    if dist <= Params.pcaMaxDist
                        cares(h).nom = noms(idx);            
                    end                
                end
            end
	    end
	end

	structCares = cares;
end

