% -----------------------------------------------------------------
% Detecci i reconeixement de cares en les imatges procedents de video o imatge estatica
% Input params:
% vid   identificador de la webcam
% handleAxes axes on es mostra el resultat del reconeixement i/o detecci
% op operaci que cal realitzar
%
% Output params:
% none
%         
% -----------------------------------------------------------------
        
function frameDisplay(obj,event,vid,handleAxes,op)	
    persistent hPlot;

    p = load('params.mat','Params');
    Params = p.Params;	
    
    %% Deteccio de cares en el video
    if strcmp(op,Params.preview) == 1
        trigger(vid);
        IM=getdata(vid,1,'uint8');
	
        imshow(IM,'Parent',handleAxes);
        
        iGray = Utils.doGray(IM);
        [faces, numFaces] = MfaceDetect.doFaceDetect(iGray,Params.scaleFaceMin);

        if numFaces > 0
            hold on;
            for h=1:size(faces,1)            
                % Es pinten els rectangles
                r = Utils.makeRect(faces(h,:));
                plot(r(:, 1), r(:, 2), 'g');                
            end    
            hold off;            
        end
    %% Reconeixement en moviment eigenfaces        
    elseif strcmp(op,Params.recEigenVid) == 1
        trigger(vid);
        IM=getdata(vid,1,'uint8');
	
        imshow(IM,'Parent',handleAxes);
        
        % Es fa el reconiexement
    	cares = eigenRecognition(IM);
            
        for h=1:size(cares,2)            
            c = cares(h).cara;
            nom = cares(h).nom;
            ue = cares(h).ue;
            ud = cares(h).ud;
            
            if size(c,1)>0                           
                rect = [c(1,1) c(1,2); 
                        c(1,1)+c(1,3) c(1,2); 
                        c(1,1)+c(1,3) c(1,2)+c(1,4); 
                        c(1,1)  c(1,2)+c(1,4); 
                        c(1,1) c(1,2)];

                hold on;
                hPlot = [hPlot plot(rect(:,1), rect(:,2), 'g')];        
                hPlot = [hPlot text(c(1,1),c(1,2)-10,nom,'color','green')];
                
                if size(ue,1)> 1
                    hPlot = [hPlot plot(ue(:,1), ue(:,2), 'b')];        
                    hPlot = [hPlot plot(ud(:,1), ud(:,2), 'b')];        
                end
                hold off;
            end        
        end
    %% Reconeixement imatge eigenfaces
    elseif strcmp(op,Params.recEigenEst) == 1        
        h = findall(handleAxes,'type','image');
        IM = get(h,'CDATA');
        
        % Es fa el reconiexement
    	cares = eigenRecognition(IM);
        
        for h=1:size(cares,2)            
            c = cares(h).cara;
            nom = cares(h).nom;
            ue = cares(h).ue;
            ud = cares(h).ud;
            
            if size(c,1)>0                           
                rect = [c(1,1) c(1,2); 
                        c(1,1)+c(1,3) c(1,2); 
                        c(1,1)+c(1,3) c(1,2)+c(1,4); 
                        c(1,1)  c(1,2)+c(1,4); 
                        c(1,1) c(1,2)];

                hold on;
                hPlot = [hPlot plot(rect(:,1), rect(:,2), 'g')];        
                hPlot = [hPlot text(c(1,1),c(1,2)-10,nom,'color','green')];
                
                if size(ue,1)> 1
                    hPlot = [hPlot plot(ue(:,1), ue(:,2), 'b')];        
                    hPlot = [hPlot plot(ud(:,1), ud(:,2), 'b')];        
                end
                hold off;
            end        
        end
    %%reconeixement en moviment fisherfaces    
    elseif strcmp(op,Params.recFisherVid) == 1
        trigger(vid);
        IM=getdata(vid,1,'uint8');
	
        imshow(IM,'Parent',handleAxes);
        
        % Es fa el reconiexement
    	cares = fisherRecognition(IM);
            
        for h=1:size(cares,2)            
            c = cares(h).cara;
            nom = cares(h).nom;
            ue = cares(h).ue;
            ud = cares(h).ud;
            
            if size(c,1)>0                           
                rect = [c(1,1) c(1,2); 
                        c(1,1)+c(1,3) c(1,2); 
                        c(1,1)+c(1,3) c(1,2)+c(1,4); 
                        c(1,1)  c(1,2)+c(1,4); 
                        c(1,1) c(1,2)];

                hold on;
                hPlot = [hPlot plot(rect(:,1), rect(:,2), 'g')];        
                hPlot = [hPlot text(c(1,1),c(1,2)-10,nom,'color','green')];
                
                if size(ue,1)> 1
                    hPlot = [hPlot plot(ue(:,1), ue(:,2), 'b')];        
                    hPlot = [hPlot plot(ud(:,1), ud(:,2), 'b')];        
                end
                hold off;
            end        
        end
    %% Reconeixement imatge fisherfaces
    elseif strcmp(op,Params.recFisherEst) == 1        
        h = findall(handleAxes,'type','image');
        IM = get(h,'CDATA');
        
        % Es fa el reconiexement
    	cares = fisherRecognition(IM);
        
        for h=1:size(cares,2)            
            c = cares(h).cara;
            nom = cares(h).nom;
            ue = cares(h).ue;
            ud = cares(h).ud;
            
            if size(c,1)>0                           
                rect = [c(1,1) c(1,2); 
                        c(1,1)+c(1,3) c(1,2); 
                        c(1,1)+c(1,3) c(1,2)+c(1,4); 
                        c(1,1)  c(1,2)+c(1,4); 
                        c(1,1) c(1,2)];

                hold on;
                hPlot = [hPlot plot(rect(:,1), rect(:,2), 'g')];        
                hPlot = [hPlot text(c(1,1),c(1,2)-10,nom,'color','green')];
                
                if size(ue,1)> 1
                    hPlot = [hPlot plot(ue(:,1), ue(:,2), 'b')];        
                    hPlot = [hPlot plot(ud(:,1), ud(:,2), 'b')];        
                end
                hold off;
            end        
        end
          
    end
end

